@@ -1,4 +1,10 @@
 package POE::Component::IRC::Plugin::BotCommand;
+BEGIN {
+  $POE::Component::IRC::Plugin::BotCommand::AUTHORITY = 'cpan:HINRIK';
+}
+BEGIN {
+  $POE::Component::IRC::Plugin::BotCommand::VERSION = '6.34';
+}
 
 use strict;
 use warnings;
@@ -6,13 +12,13 @@ use Carp;
 use POE::Component::IRC::Common qw( parse_user strip_color strip_formatting );
 use POE::Component::IRC::Plugin qw( :ALL );
 
-our $VERSION = '6.32';
-
 sub new {
     my ($package) = shift;
     croak "$package requires an even number of arguments" if @_ & 1;
     my %args = @_;
-    
+
+    $args{Method} = 'notice' if !defined $args{Method};
+
     for my $cmd (keys %{ $args{Commands} }) {
         $args{Commands}->{lc $cmd} = delete $args{Commands}->{$cmd};
     }
@@ -22,10 +28,10 @@ sub new {
 sub PCI_register {
     my ($self, $irc) = splice @_, 0, 2;
     
-    $self->{Addressed}     = 1 if !defined $self->{Addressed};
-    $self->{Prefix}        = '!' if !defined $self->{Prefix};
-    $self->{In_channels}   = 1 if !defined $self->{In_channels};
-    $self->{In_private}    = 1 if !defined $self->{In_private};
+    $self->{Addressed}   = 1   if !defined $self->{Addressed};
+    $self->{Prefix}      = '!' if !defined $self->{Prefix};
+    $self->{In_channels} = 1   if !defined $self->{In_channels};
+    $self->{In_private}  = 1   if !defined $self->{In_private};
     $self->{irc} = $irc;
     
     $irc->plugin_register( $self, 'SERVER', qw(msg public) );
@@ -46,16 +52,12 @@ sub S_msg {
     $what = $self->_normalize($what);
     
     my ($cmd, $args);
-    if (!(($cmd, $args) = $what =~ /^(\w+)(?:\s+(.+))?$/)) {
+    if (!(($cmd, $args) = $what =~ /^$self->{Prefix}(\w+)(?:\s+(.+))?$/)) {
         return PCI_EAT_NONE;
     }
     
-    my $handled = $self->_handle_cmd($who, $where, $cmd, $args);
-    
-    if ($self->{Eat} && $handled) {
-        return PCI_EAT_PLUGIN;
-    }
-    return PCI_EAT_NONE;
+    $self->_handle_cmd($who, $where, $cmd, $args);
+    return $self->{Eat} ? PCI_EAT_PLUGIN : PCI_EAT_NONE;
 }
 
 sub S_public {
@@ -79,13 +81,9 @@ sub S_public {
     if (!(($cmd, $args) = $what =~ /^(\w+)(?:\s+(.+))?$/)) {
         return PCI_EAT_NONE;
     }
- 
-    my $handled = $self->_handle_cmd($who, $where, $cmd, $args);
-    
-    if ($self->{Eat} && $handled) {
-        return PCI_EAT_PLUGIN;
-    }
-    return PCI_EAT_NONE;
+
+    $self->_handle_cmd($who, $where, $cmd, $args);
+    return $self->{Eat} ? PCI_EAT_PLUGIN : PCI_EAT_NONE;
 }
 
 sub _normalize {
@@ -98,27 +96,45 @@ sub _normalize {
 sub _handle_cmd {
     my ($self, $who, $where, $cmd, $args) = @_;
     my $irc = $self->{irc};
+    my $chantypes = join('', @{ $irc->isupport('CHANTYPES') || ['#', '&']});
+    my $public = $where =~ /^[$chantypes]/ ? 1 : 0;
     $cmd = lc $cmd;
 
+    if (ref $self->{Auth_sub} eq 'CODE') {
+        my ($authed, $errors) = $self->{Auth_sub}->($self->{irc}, $who, $where, $cmd, $args);
+
+        if (!$authed) {
+            my @errors = ref $errors eq 'ARRAY'
+                ? @$errors
+                : 'You are not authorized to use this command.';
+            for my $error (@errors) {
+                $irc->yield($self->{Method}, $where, $error);
+            }
+            return;
+        }
+    }
+
     if (defined $self->{Commands}->{$cmd}) {
         $irc->send_event("irc_botcmd_$cmd" => $who, $where, $args);
     }
     elsif ($cmd =~ /^help$/i) {
-        my @help = $self->_get_help($args);
-        $irc->yield(notice => $where => $_) for @help;
+        my @help = $self->_get_help($args, $public);
+        $irc->yield($self->{Method} => $where => $_) for @help;
     }
-    else {
-        return if $self->{Ignore_unknown};
-        my @help = $self->_get_help($cmd);
-        $irc->yield(notice => $where => $_) for @help;
+    elsif (!$self->{Ignore_unknown}) {
+        my @help = $self->_get_help($cmd, $public);
+        $irc->yield($self->{Method} => $where => $_) for @help;
     }
 
-    return 1;
+    return;
 }
 
 sub _get_help {
-    my ($self, $args) = @_;
+    my ($self, $args, $public) = @_;
     my $irc = $self->{irc};
+    my $p = $self->{Addressed} && $public
+        ? $irc->nick_name().': '
+        : $self->{Prefix};
     
     my @help;
     if (defined $args) {
@@ -128,13 +144,13 @@ sub _get_help {
         }
         else {
             push @help, "Unknown command: $cmd";
-            push @help, 'To get a list of commands, use: /msg '. $irc->nick_name() . ' help';
+            push @help, "To get a list of commands, use: ${p}help";
         }
     }
     else {
         if (keys %{ $self->{Commands} }) {
             push @help, 'Commands: ' . join ', ', keys %{ $self->{Commands} };
-            push @help, 'For more details, use: /msg ' . $irc->nick_name() . ' help <command>';
+            push @help, "For more details, use: ${p}help <command>";
         }
         else {
             push @help, 'No commands are defined';
@@ -271,12 +287,12 @@ command yourself, that one will be used instead.
 
 =head2 C<new>
 
-Four optional arguments:
-
 B<'Commands'>, a hash reference, with your commands as keys, and usage
 information as values. If the usage string contains newlines, the component
 will send one message for each line.
 
+=head3 Accepting commands
+
 B<'In_channels'>, a boolean value indicating whether to accept commands in
 channels. Default is true.
 
@@ -286,16 +302,53 @@ private. Default is true.
 B<'Addressed'>, requires users to address the bot by name in order
 to issue commands. Default is true.
 
-B<'Prefix'>, if B<'Addressed'> is false, all channel commands must be prefixed
-with this string. Default is '!'. You can set it to '' to allow bare channel
-commands.
+B<'Prefix'>, a string which all commands must be prefixed with (except in
+channels when B<'Addressed'> is true). Default is '!'. You can set it to ''
+to allow bare commands.
+
+=head3 Authorization
+
+B<'Auth_sub'>, a subroutine reference which, if provided, will be called
+for every command. The subroutine will be called in list context. If the
+first value returned is true the command will be processed as normal. If the
+value is false, then no events will be generated, and an error message will
+possibly be sent back to the user. You can override the default error message
+by returning an array reference of (zero or more) strings. Each string will
+be sent as a message to the user.
+
+The sub will get the following arguments:
+
+=over
+
+=item 1. The IRC component object
+
+=item 2. The nick!user@host of the user
+
+=item 3. The place where the command was issued (the nickname of the user if
+it was in private)
+
+=item 4. The name of the command
+
+=item 5. The command argument string
+
+=back
+
+B<'Ignore_unauthorized'>, if true, the plugin will ignore unauthorized
+commands, rather than printing an error message upon receiving them. This is
+only relevant if B<'Auth_sub'> is also supplied. Default is false.
+
+=head3 Miscellaneous
 
 B<'Ignore_unknown'>, if true, the plugin will ignore undefined commands,
 rather than printing a help message upon receiving them. Default is false.
 
+B<'Method'>, how you want help messages to be delivered. Valid options are
+'notice' (the default) and 'privmsg'.
+
 B<'Eat'>, set to true to make the plugin hide
-L<C<irc_public>|POE::Component::IRC/"irc_public"> events from other plugins if
-they contain a valid command. Default is false.
+L<C<irc_public>|POE::Component::IRC/"irc_public"> events from other plugins
+when they look like commands. Probably only useful when a B<'Prefix'> is
+defined. Default is false.
 
 Returns a plugin object suitable for feeding to
 L<POE::Component::IRC|POE::Component::IRC>'s C<plugin_add> method.
@@ -328,36 +381,6 @@ command was issued, or the sender's nickname if this was a private message.
 If the command was followed by any arguments, C<ARG2> will be a string
 containing them, otherwise it will be undefined.
 
-=head1 TODO
-
-Add permissions/authorization. E.g. allow the user to specify if commands are
-only available ops, or only to users matching some IRC masks, etc.
-
-It would have to support permissions/auth on a per-command level, so that
-a bot can get by with a single BotCommand plugin, with respect to easily
-listing the available commands in a help message. Maybe augmenting the
-C<add()> method to accept an optional hash reference argument detailing
-authorization requirements is appropriate here. I suppose plugins that call
-C<add()> to add new commands should accept a hash reference like that as an
-B<'auth'> argument to their constructor.
-
-I considered having the auth settings apply to all commands, and using
-multiple BotCommand plugins to group commands by who is allowed to issue them,
-but this approach is more complex if we want the bot to complain about
-undefined commands, or when someone wants a list of all commands. Plugins
-which define new commands would accept a B<'botcmd'> parameter to choose which
-BotCommand plugin they should call C<add()>/C<remove()> on.
-
-Some prior art to consider:
-
-=over 4
-
-=item L<POE::Component::IRC::Plugin::BaseWrap|POE::Component::IRC::PluginBaseWrap>
-
-=item L<Bot::BasicBot::Pluggable::Module::Auth|Bot::BasicBot::Pluggable::Module::Auth> 
-
-=back
-
 =head1 AUTHOR
 
 Hinrik E<Ouml>rn SigurE<eth>sson, hinrik.sig@gmail.com
